"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TeleBot = exports.DEFAULT_DEBUG = exports.DEFAULT_POLLING = void 0;
const axios_1 = __importDefault(require("axios"));
const form_data_1 = __importDefault(require("form-data"));
const fs_1 = require("fs");
const errors_1 = require("./errors");
const logger_1 = require("./telebot/logger");
const processors_1 = require("./telebot/processors");
const webhook_1 = require("./telebot/webhook");
const utils_1 = require("./utils");
const TELEGRAM_BOT_API = (botToken) => `https://api.telegram.org/bot${botToken}`;
exports.DEFAULT_POLLING = {
    offset: 0,
    interval: false,
    timeout: 0,
    limit: 100,
    retryTimeout: 3000,
    retryTimes: Infinity
};
exports.DEFAULT_DEBUG = {
    levels: Object.values(logger_1.Levels).filter(value => typeof value === "number")
};
class TeleBot {
    constructor(options) {
        this.polling = exports.DEFAULT_POLLING;
        this.debug = false;
        this.flags = {
            isRunning: false,
            canFetch: true,
            waitEvents: false
        };
        this.events = new Map();
        this.hearsEvents = new Map();
        this.modifiers = new Map();
        this.plugins = new Map();
        if (typeof options === "string") {
            options = {
                token: options
            };
        }
        const { token, botAPI, polling, webhook, allowedUpdates, log } = options;
        this.botToken = token;
        if (!this.botToken) {
            throw new errors_1.TeleBotError("Telegram bot token is not provided.");
        }
        this.botAPI = botAPI ? botAPI(this.botToken) : TELEGRAM_BOT_API(this.botToken);
        this.webhook = webhook;
        if (polling) {
            const { offset, interval, timeout, limit, retryTimes, retryTimeout } = polling;
            if (offset !== undefined && Number.isInteger(offset)) {
                this.setOffset(offset);
            }
            if (interval !== undefined && interval > 0) {
                this.polling.interval = interval;
            }
            if (timeout !== undefined && timeout > 0) {
                this.polling.timeout = timeout;
            }
            if (limit !== undefined && limit > 0) {
                this.polling.limit = limit;
            }
            if (retryTimes !== undefined && Number.isInteger(retryTimes)) {
                this.polling.retryTimes = retryTimes;
            }
            if (retryTimeout !== undefined && Number.isInteger(retryTimeout)) {
                this.polling.retryTimeout = retryTimeout;
            }
        }
        else if (webhook) {
            const { url } = webhook;
            const { port: urlPort, protocol } = utils_1.parseUrl(url);
            const webhookPort = urlPort ? parseInt(urlPort) : 80;
            if (!(protocol === null || protocol === void 0 ? void 0 : protocol.toLowerCase().startsWith("https"))) {
                throw new errors_1.TeleBotError(`Webhook '${url}' must be HTTPS secured.`);
            }
            if (!webhook_1.ALLOWED_WEBHOOK_PORTS.includes(webhookPort)) {
                throw new errors_1.TeleBotError(`Webhook port '${webhookPort}' is not supported by Telegram Bot API. Use allowed webhook ports: ${webhook_1.ALLOWED_WEBHOOK_PORTS.join(", ")}`);
            }
            // if (host) {
            //     throw new TeleBotError("Webhook host is required.");
            // }
            //
            // if (!port || !Number.isInteger(port) || port <= 0) {
            //     throw new TeleBotError(`Invalid webhook port: '${port}'`);
            // }
        }
        if (allowedUpdates !== undefined && Array.isArray(allowedUpdates) && allowedUpdates.length) {
            this.allowedUpdates = allowedUpdates;
        }
        if (log) {
            this.debug = this.debug !== true ? log : exports.DEFAULT_DEBUG;
        }
        else if (log === false) {
            this.debug = false;
        }
        this.logger = new logger_1.TeleBotLogger("telebot", this.debug);
        this.logger.debug(logger_1.LID.Initial, {
            meta: {
                options: this.getOptions()
            }
        });
    }
    getToken() {
        return this.botToken;
    }
    getOptions() {
        return {
            token: this.botToken,
            botAPI: this.botAPI,
            polling: this.polling,
            webhook: this.webhook,
            allowedUpdates: this.allowedUpdates
        };
    }
    start() {
        return __awaiter(this, void 0, void 0, function* () {
            const { interval } = this.polling;
            const webhook = this.webhook;
            if (!this.hasFlag("isRunning")) {
                this.setFlag("isRunning");
                try {
                    const deleteWebhook = () => __awaiter(this, void 0, void 0, function* () { return yield this.deleteWebhook(); });
                    this.me = yield this.getMe();
                    if (webhook) {
                        const url = `${webhook.url}/${this.getToken()}`;
                        yield this.setWebhook(url, {
                            certificate: webhook.cert
                        });
                        if (webhook.host && webhook.port) {
                            yield webhook_1.webhookServer(this, webhook);
                        }
                    }
                    else if (interval && interval > 0) {
                        yield deleteWebhook();
                        this.startLifeInterval(interval);
                    }
                    else if (interval === false) {
                        yield deleteWebhook();
                        this.startLifeCycle();
                    }
                }
                catch (e) {
                    const error = errors_1.normalizeError(e);
                    this.dispatch("error", error);
                    this.logger.error(logger_1.LID.TeleBot, { error });
                    // eslint-disable-next-line no-console
                    console.error("==== TELEBOT GLOBAL ERROR ====", error);
                    return Promise.reject(error);
                }
            }
            else {
                yield this.stop();
                throw new errors_1.TeleBotError(errors_1.ERROR_TELEBOT_ALREADY_RUNNING);
            }
        });
    }
    restart() {
        return __awaiter(this, void 0, void 0, function* () {
            this.logger.info(logger_1.LID.Restart);
            yield this.stop();
            return this.start();
        });
    }
    stop() {
        return __awaiter(this, void 0, void 0, function* () {
            this.logger.info(logger_1.LID.Stop);
            this.unsetFlag("isRunning");
            if (this.lifeIntervalFn) {
                clearInterval(this.lifeIntervalFn);
                this.lifeIntervalFn = undefined;
            }
        });
    }
    getFlags() {
        return this.flags;
    }
    hasFlag(name) {
        return this.flags[name];
    }
    setFlag(name) {
        this.logger.debug(logger_1.LID.SetFlag, { meta: { flag: name } });
        this.flags[name] = true;
    }
    unsetFlag(name) {
        this.logger.debug(logger_1.LID.UnsetFlag, { meta: { flag: name } });
        this.flags[name] = false;
    }
    setOffset(offset) {
        this.logger.debug(logger_1.LID.SetOffset, { meta: { offset } });
        this.polling.offset = offset;
    }
    startLifeCycle() {
        this.logger.info(logger_1.LID.StartLifeCycle);
        this.lifeCycle(false).catch((error) => {
            this.logger.error(logger_1.LID.StartLifeCycle, { error });
            this.dispatch("error", error);
        });
    }
    startLifeInterval(interval) {
        this.logger.info(logger_1.LID.StartLifeInterval, {
            meta: { interval }
        });
        this.lifeIntervalFn = setInterval(() => {
            this.logger.debug(logger_1.LID.LiveInterval);
            if (this.hasFlag("isRunning") && this.hasFlag("canFetch")) {
                this.unsetFlag("canFetch");
                this.lifeCycle(true).then(() => {
                    this.setFlag("canFetch");
                }).catch((error) => {
                    this.dispatch("error", error);
                    this.logger.error(logger_1.LID.StartLifeInterval, { error });
                });
            }
        }, interval);
    }
    lifeCycle(liveOnce = true) {
        this.logger.debug(logger_1.LID.Tick);
        if (this.hasFlag("isRunning")) {
            return this.fetchTelegramUpdates().then(() => {
                return liveOnce ? Promise.resolve() : this.lifeCycle(false);
            }).catch((e) => {
                const error = errors_1.handleTelegramResponse(e);
                this.logger.error(logger_1.LID.LifeCycle, { error });
                return Promise.reject(error);
            });
        }
        else {
            this.logger.debug(logger_1.LID.LifeCycle, "not running");
            return Promise.resolve();
        }
    }
    fetchTelegramUpdates(offset = this.polling.offset, limit = this.polling.limit, timeout = this.polling.timeout) {
        this.logger.debug(logger_1.LID.FetchTelegramUpdates, {
            meta: { offset, limit, timeout }
        });
        const promise = this.telegramRequest("getUpdates", { offset, limit, timeout });
        return promise.then((updates) => {
            let processPromise = Promise.resolve();
            this.logger.debug(logger_1.LID.FetchTelegramUpdates, {
                meta: { updates }
            });
            if (updates) {
                processPromise = this.processTelegramUpdates(updates);
            }
            return processPromise.then(() => {
                if (updates && updates.length > 0) {
                    this.setOffset(++updates[updates.length - 1].update_id);
                }
            });
        }).catch((e) => {
            const error = errors_1.normalizeError(e);
            this.logger.error(logger_1.LID.FetchTelegramUpdates, { error });
            return Promise.reject(error);
        });
    }
    processTelegramUpdates(updates) {
        return __awaiter(this, void 0, void 0, function* () {
            const processorPromises = [];
            this.logger.info(logger_1.LID.ProcessTelegramUpdates, {
                meta: { updates }
            });
            for (const update of updates) {
                for (const name of Object.keys(processors_1.TELEGRAM_UPDATE_PROCESSORS)) {
                    const processorName = name;
                    if (processorName in update) {
                        const updatePayload = yield this.run(processorName, update[processorName]);
                        if (updatePayload) {
                            processorPromises.push(processors_1.TELEGRAM_UPDATE_PROCESSORS[processorName].call(this, updatePayload, {}));
                        }
                        break;
                    }
                }
            }
            return this.flags.waitEvents ? Promise.all(processorPromises) : Promise.resolve(processorPromises);
        });
    }
    telegramRequest(endpoint, payload) {
        const url = `${this.botAPI}/${endpoint}`;
        this.logger.info(logger_1.LID.TelegramRequest, {
            meta: { endpoint, payload }
        });
        return axios_1.default.request({
            url,
            headers: payload instanceof form_data_1.default ? payload.getHeaders() : undefined,
            data: payload,
            method: "post",
            responseType: "json",
            maxContentLength: payload instanceof form_data_1.default ? Infinity : undefined,
            proxy: this.proxy
        })
            .then((response) => {
            const { ok, result } = response.data;
            this.logger.http(logger_1.LID.TelegramRequest, {
                meta: { response: response.data }
            });
            if (!ok || result === undefined) {
                return Promise.reject(response.data);
            }
            return result;
        })
            .catch((e) => {
            const error = errors_1.handleTelegramResponse(e);
            this.logger.error(logger_1.LID.TelegramRequest, { error });
            return Promise.reject(error);
        });
    }
    telegramMethod({ method, required = {}, optional = {}, isDataForm }) {
        return __awaiter(this, void 0, void 0, function* () {
            const { required: modifiedRequired, optional: modifiedOptional } = yield this.run(method, { required, optional }, { method, required, optional, isDataForm });
            let payload = Object.assign({}, modifiedRequired, modifiedOptional);
            if (isDataForm) {
                const form = new form_data_1.default();
                for (const [key, value] of Object.entries(payload)) {
                    let payload = value;
                    if (Array.isArray(value)) {
                        const data = value.map((inputMedia) => {
                            if (inputMedia.media instanceof fs_1.ReadStream) {
                                const fileId = utils_1.randomString();
                                form.append(fileId, inputMedia.media);
                                inputMedia.media = `attach://${fileId}`;
                            }
                            return inputMedia;
                        });
                        payload = JSON.stringify(data);
                    }
                    form.append(key, payload);
                }
                payload = form;
            }
            this.logger.debug(logger_1.LID.TelegramMethod, {
                meta: { method, required, optional }
            });
            return this.telegramRequest(method, payload);
        });
    }
    uploadFile(path) {
        return fs_1.createReadStream(path);
    }
    parallel(tasks) {
        return Promise.all(tasks);
    }
    plugin(plugin, context) {
        this.logger.info(logger_1.LID.Plugin, { meta: plugin });
        if (!this.plugins.get(plugin.id)) {
            this.plugins.set(plugin.id, {
                id: plugin.id,
                name: plugin.name,
                version: plugin.version,
                author: plugin.author,
                description: plugin.description,
                homepage: plugin.homepage
            });
            plugin.plugin.call(this, this, context);
        }
    }
    hears(text, processor) {
        this.logger.debug(logger_1.LID.Hears, {
            meta: { on: text }
        });
        for (const value of utils_1.convertToArray(text)) {
            const handler = this.hearsEvents.get(value);
            if (handler) {
                handler.processors.add(processor);
            }
            else {
                this.hearsEvents.set(value, {
                    processors: new Set([processor])
                });
            }
        }
    }
    dispatchHears(text, payload) {
        this.logger.debug(logger_1.LID.Hears, {
            meta: { dispatch: text }
        });
        const eventPromise = [];
        for (const textValue of utils_1.convertToArray(text)) {
            for (const hearsEvent of this.hearsEvents.keys()) {
                let match = textValue;
                if (hearsEvent instanceof RegExp) {
                    match = textValue.match(hearsEvent);
                    if (!match) {
                        continue;
                    }
                }
                else if (hearsEvent !== textValue) {
                    continue;
                }
                const eventHandler = this.hearsEvents.get(hearsEvent);
                for (const processor of eventHandler.processors) {
                    eventPromise.push(new Promise((resolve) => {
                        try {
                            const processorOutput = processor.call(this, payload, { match: match });
                            if (processorOutput instanceof Promise) {
                                processorOutput.then(resolve).catch((e) => {
                                    const error = errors_1.normalizeError(e, payload);
                                    this.logger.error(logger_1.LID.DispatchHears, { error });
                                    this.dispatch("error", error, { methodName: textValue });
                                    resolve(error);
                                });
                            }
                            else {
                                resolve(processorOutput);
                            }
                        }
                        catch (e) {
                            const error = errors_1.normalizeError(e, payload);
                            this.logger.error(logger_1.LID.DispatchHears, { error });
                            this.dispatch("error", error, { methodName: textValue });
                            resolve(error);
                        }
                    }));
                }
            }
        }
        return this.flags.waitEvents ? Promise.all(eventPromise) : Promise.resolve(eventPromise);
    }
    mod(name, processor) {
        this.logger.debug(logger_1.LID.Modifier, {
            meta: { name }
        });
        const eventHandler = this.modifiers.get(name);
        if (eventHandler) {
            eventHandler.processors.add(processor);
        }
        else {
            this.modifiers.set(name, {
                processors: new Set([processor])
            });
        }
    }
    run(event, payload, context) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logger.debug(logger_1.LID.Modifier, {
                meta: { run: event }
            });
            const handler = this.modifiers.get(event);
            if (handler) {
                for (const processor of handler.processors) {
                    payload = yield processor.call(this, payload, context);
                }
            }
            return payload;
        });
    }
    on(event, processor) {
        this.logger.debug(logger_1.LID.Event, {
            meta: { on: event }
        });
        for (const eventName of utils_1.convertToArray(event)) {
            const eventHandler = this.events.get(eventName);
            if (eventHandler) {
                eventHandler.processors.add(processor);
            }
            else {
                this.events.set(eventName, {
                    processors: new Set([processor])
                });
            }
        }
    }
    dispatch(event, payload, context = {}) {
        this.logger.debug(logger_1.LID.Event, {
            meta: { dispatch: event }
        });
        const eventPromise = [];
        for (const eventName of utils_1.convertToArray(event)) {
            const eventHandler = this.events.get(eventName);
            if (!eventHandler) {
                continue;
            }
            const eventContext = Object.assign({ eventName }, context);
            for (const processor of eventHandler.processors) {
                eventPromise.push(new Promise((resolve) => {
                    try {
                        const processorOutput = processor.call(this, payload, eventContext);
                        if (processorOutput instanceof Promise) {
                            processorOutput.then(resolve).catch((e) => {
                                const error = errors_1.normalizeError(e, payload);
                                this.logger.error(logger_1.LID.Dispatch, { error });
                                if (eventName !== "error") {
                                    this.dispatch("error", error, eventContext);
                                }
                                else {
                                    // TODO: Error in error processor
                                    console.log("ASYNC EVENT ERROR", error, eventContext);
                                }
                                resolve(error);
                            });
                        }
                        else {
                            resolve(processorOutput);
                        }
                    }
                    catch (e) {
                        const error = errors_1.normalizeError(e, payload);
                        this.logger.error(logger_1.LID.Dispatch, { error });
                        if (eventName !== "error") {
                            this.dispatch("error", error, eventContext);
                        }
                        else {
                            // TODO: Error in error processor
                            console.log("SYNC EVENT ERROR", error, eventContext);
                        }
                        resolve(error);
                    }
                }));
            }
        }
        return this.flags.waitEvents ? Promise.all(eventPromise) : Promise.resolve(eventPromise);
    }
}
exports.TeleBot = TeleBot;
