import { CallbackQuery, ChatMemberUpdated, ChosenInlineResult, InlineQuery, Message, Poll, PollAnswer, PreCheckoutQuery, ShippingQuery, TelegramBotToken, TelegramMessageNames, TelegramUpdateNames } from "./telegram";
import { TeleBotLogOptions } from "../telebot/logger";
import { SomeKindOfError } from "../errors";
import { TelegramUpdateKeys } from "../telebot/processors";
import { AxiosProxyConfig } from "axios";
import { TeleBot } from "../telebot";
export declare type TeleBotOptions = {
    token: TelegramBotToken;
    botAPI?: (botToken: string) => string;
    polling?: Partial<TeleBotPolling>;
    webhook?: WebhookOptions;
    allowedUpdates?: TelegramUpdateNames[];
    proxy?: AxiosProxyConfig;
    log?: boolean | TeleBotLogOptions;
};
export declare type WebhookOptions = {
    url: string;
    host?: string;
    port?: number;
    key?: string;
    cert?: string;
};
export declare type TeleBotFlags = {
    isRunning: boolean;
    canFetch: boolean;
    waitEvents: boolean;
};
export declare type TeleBotPolling = {
    offset: number;
    interval: number | false;
    timeout: number;
    limit: number;
    retryTimes: number;
    retryTimeout: number;
};
export declare enum TeleBotScenario {
    Reconnect = "reconnect",
    Restart = "restart",
    Terminate = "terminate",
    Pass = "pass"
}
export declare type TeleBotRunningInstanceScenario = TeleBotScenario.Restart | TeleBotScenario.Terminate | TeleBotScenario.Pass;
export declare type TelegramFetchErrorScenario = TeleBotScenario.Reconnect | TeleBotScenario.Terminate;
declare type TeleBotEventMessageUpdate<T extends TelegramMessageNames> = Omit<Message, TelegramMessageNames> & Required<Pick<Message, T>>;
export declare type TeleBotEventNames = {
    text: TeleBotEventMessageUpdate<"text">;
    audio: TeleBotEventMessageUpdate<"audio">;
    document: TeleBotEventMessageUpdate<"document">;
    animation: TeleBotEventMessageUpdate<"animation">;
    game: TeleBotEventMessageUpdate<"game">;
    photo: TeleBotEventMessageUpdate<"photo">;
    sticker: TeleBotEventMessageUpdate<"sticker">;
    video: TeleBotEventMessageUpdate<"video">;
    video_note: TeleBotEventMessageUpdate<"video_note">;
    voice: TeleBotEventMessageUpdate<"voice">;
    caption: TeleBotEventMessageUpdate<"caption">;
    contact: TeleBotEventMessageUpdate<"contact">;
    location: TeleBotEventMessageUpdate<"location">;
    venue: TeleBotEventMessageUpdate<"venue">;
    poll: TeleBotEventMessageUpdate<"poll">;
    dice: TeleBotEventMessageUpdate<"dice">;
    new_chat_members: TeleBotEventMessageUpdate<"new_chat_members">;
    left_chat_member: TeleBotEventMessageUpdate<"left_chat_member">;
    new_chat_title: TeleBotEventMessageUpdate<"new_chat_title">;
    new_chat_photo: TeleBotEventMessageUpdate<"new_chat_photo">;
    delete_chat_photo: TeleBotEventMessageUpdate<"delete_chat_photo">;
    group_chat_created: TeleBotEventMessageUpdate<"group_chat_created">;
    supergroup_chat_created: TeleBotEventMessageUpdate<"supergroup_chat_created">;
    channel_chat_created: TeleBotEventMessageUpdate<"channel_chat_created">;
    message_auto_delete_timer_changed: TeleBotEventMessageUpdate<"message_auto_delete_timer_changed">;
    migrate_to_chat_id: TeleBotEventMessageUpdate<"migrate_to_chat_id">;
    migrate_from_chat_id: TeleBotEventMessageUpdate<"migrate_from_chat_id">;
    pinned_message: TeleBotEventMessageUpdate<"pinned_message">;
    invoice: TeleBotEventMessageUpdate<"invoice">;
    successful_payment: TeleBotEventMessageUpdate<"successful_payment">;
    connected_website: TeleBotEventMessageUpdate<"connected_website">;
    passport_data: TeleBotEventMessageUpdate<"passport_data">;
    proximity_alert_triggered: TeleBotEventMessageUpdate<"proximity_alert_triggered">;
    voice_chat_started: TeleBotEventMessageUpdate<"voice_chat_started">;
    voice_chat_ended: TeleBotEventMessageUpdate<"voice_chat_ended">;
    voice_chat_participants_invited: TeleBotEventMessageUpdate<"voice_chat_participants_invited">;
    message: Message;
    edited_message: Message;
    channel_post: Message;
    edited_channel_post: Message;
    inline_query: InlineQuery;
    chosen_inline_result: ChosenInlineResult;
    callback_query: CallbackQuery;
    shipping_query: ShippingQuery;
    pre_checkout_query: PreCheckoutQuery;
    poll_update: Poll;
    poll_answer: PollAnswer;
    my_chat_member: ChatMemberUpdated;
    chat_member: ChatMemberUpdated;
    error: SomeKindOfError;
};
export declare type TeleBotHearsName = string | RegExp;
export declare type TeleBotHearsContext<T extends TeleBotHearsName> = {
    match: T extends RegExp ? RegExpMatchArray : string;
};
export declare type TeleBotHearsPayload = TeleBotEventNames["text"];
export declare type TeleBotHearsProcessor<T extends TeleBotHearsName> = (payload: TeleBotHearsPayload, context: TeleBotHearsContext<T>) => Promise<unknown> | unknown;
export declare type TeleBotHears = {
    processors: Set<TeleBotHearsProcessor<TeleBotHearsName>>;
};
export declare type TeleBotEventName = keyof TeleBotEventNames;
export declare type TeleBotEventContext = Record<string, unknown>;
export declare type TeleBotEventPayload<T extends TeleBotEventName> = TeleBotEventNames[T];
export declare type TeleBotEventProcessor<T extends TeleBotEventName> = (payload: TeleBotEventNames[T], context?: TeleBotEventContext) => Promise<unknown> | unknown;
export declare type TeleBotEvent = {
    processors: Set<TeleBotEventProcessor<any>>;
};
export declare type TeleBotModifierName = TeleBotMethodName | keyof TeleBotEventNames | keyof TelegramUpdateKeys;
export declare type TeleBotModifierContext = Record<string, unknown>;
export declare type TeleBotModifierPayload = Record<string, unknown>;
export declare type TeleBotModifierProcessor<T extends TeleBotModifierName, P = any> = (payload: P, context?: TeleBotModifierContext) => Promise<P> | P;
export declare type TeleBotModifier = {
    processors: Set<TeleBotModifierProcessor<TeleBotModifierName>>;
};
export declare type TeleBotMethodName = "getMe" | "logOut" | "close" | "sendMessage" | "forwardMessage" | "copyMessage" | "sendPhoto" | "sendAudio" | "sendDocument" | "sendVideo" | "sendAnimation" | "sendVoice" | "sendVideoNote" | "sendMediaGroup" | "sendLocation" | "editMessageLiveLocation" | "stopMessageLiveLocation" | "sendVenue" | "sendContact" | "sendPoll" | "sendDice" | "sendChatAction" | "getUserProfilePhotos" | "getFile" | "getFileUrl" | "kickChatMember" | "unbanChatMember" | "restrictChatMember" | "promoteChatMember" | "setChatAdministratorCustomTitle" | "exportChatInviteLink" | "createChatInviteLink" | "editChatInviteLink" | "revokeChatInviteLink" | "setChatPhoto" | "deleteChatPhoto" | "setChatTitle" | "setChatDescription" | "pinChatMessage" | "unpinChatMessage" | "unpinAllChatMessages" | "leaveChat" | "getChat" | "getChatAdministrators" | "getChatMembersCount" | "getChatMember" | "setChatStickerSet" | "deleteChatStickerSet" | "answerCallbackQuery" | "setMyCommands" | "getMyCommands" | "editMessageText" | "editMessageCaption" | "editMessageMedia" | "editMessageReplyMarkup" | "stopPoll" | "deleteMessage" | "sendSticker" | "getStickerSet" | "uploadStickerFile" | "createNewStickerSet" | "addStickerToSet" | "setStickerPositionInSet" | "setStickerSetThumb" | "answerInlineQuery" | "sendInvoice" | "answerShippingQuery" | "answerPreCheckoutQuery" | "setPassportDataErrors" | "sendGame" | "setGameScore" | "getGameHighScores" | "setWebhook" | "deleteWebhook" | "getWebhookInfo" | "inlineKeyboard" | "replyKeyboard" | "replyKeyboardRemove" | "forceReply";
export declare type TeleBotPluginContext = Record<string, unknown>;
export declare type TeleBotPlugin<T = TeleBotPluginContext> = {
    id: string;
    name: string;
    version: string;
    author?: string;
    description?: string;
    homepage?: string;
    plugin(this: TeleBot, bot: TeleBot, context: T): void;
};
export {};
